<?php

namespace Drupal\smart_string_replacer\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\node\Entity\Node;

/**
 * Setting form for smart_string_replacer.
 */
class SmartStringReplacerSettingsForm extends FormBase
{

  /**
   * {@inheritdoc}
   */
  public function getFormId()
  {
    return 'smart_string_replacer_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state)
  {
    // Fetching all nodes for selection.
    $nodes = Node::loadMultiple();
    $options = [];
    foreach ($nodes as $node) {
      $options[$node->id()] = $node->getTitle();
    }

    $form['node'] = [
      '#type' => 'select',
      '#title' => $this->t('Select Nodes'),
      '#options' => $options,
      '#multiple' => TRUE,
      '#required' => TRUE,
    ];

    $form['find'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Find'),
      '#maxlength' => 30,
      '#required' => TRUE,
    ];

    $form['replace'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Replace'),
      '#maxlength' => 30,
      '#required' => TRUE,
    ];

    $form['case_sensitive'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Case sensitive'),
      '#default_value' => FALSE,
    ];

    $form['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Find and Replace'),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state)
  {
    $node_ids = $form_state->getValue('node');
    $find = $form_state->getValue('find');
    $replace = $form_state->getValue('replace');
    $case_sensitive = $form_state->getValue('case_sensitive');

    $replaced_count = 0;

    foreach ($node_ids as $node_id) {
      $node = Node::load($node_id);
      if ($node) {
        // Assuming 'body' is the correct field name.
        // Update if necessary.
        $field_name = 'body';

        // Load the existing content and its format.
        $field = $node->get($field_name);
        $content = $field->value;
        $format = $field->format;

        // Perform a find-and-replace based on case sensitivity.
        if ($case_sensitive) {
          $new_content = str_replace($find, $replace, $content);
        } else {
          $new_content = str_ireplace($find, $replace, $content);
        }

        // If content changed, save the node.
        if ($new_content !== $content) {
          $node->set($field_name, [
            'value' => $new_content,
            'format' => $format,
          ]);
          $node->save();
          $replaced_count++;
        }
      }
    }

    $this->messenger()->addMessage($this->t(
      'Replaced "@find" with "@replace" in @count node(s).',
      [
        '@find' => $find,
        '@replace' => $replace,
        '@count' => $replaced_count,
      ]
    ));
  }
}
